<?php
namespace BwlFaqManager\Helpers;

/**
 * Class for plugin constants.
 *
 * @package BwlFaqManager
 */
class PluginConstants {

	/**
     * Static property to hold plugin options.
     *
     * @var array
     */
    public static $plugin_options = [];

    /**
     * Initialize the plugin options.
     */
    public static function init() {

				define( 'BAF_OPTIONS_ID', 'bwl_advanced_faq_options' );
				self::$plugin_options = get_option( BAF_OPTIONS_ID );

				// Plugin Installation Date Time.
				define( 'BAF_INSTALLATION_DATE', 'baf_installation_date' );

		if ( empty( get_option( BAF_INSTALLATION_DATE ) ) ) {
			update_option( BAF_INSTALLATION_DATE, date( 'Y-m-d H:i:s' ) );
		}
    }

		/**
         * Get the absolute path to the plugin root.
         *
         * @return string
         * @example wp-content/plugins/bwl-advanced-faq-manager/
         */
    public static function get_plugin_path(): string {
        return dirname( dirname( __DIR__ ) ) . '/';
    }


    /**
     * Get the plugin URL.
     *
     * @return string
     * @example http://bafwp.local/wp-content/plugins/bwl-advanced-faq-manager/
     */
    public static function get_plugin_url(): string {
		return plugin_dir_url( self::get_plugin_path() . BAF_ROOT_FILE );
	}

	/**
	 * Register the plugin constants.
	 */
	public static function register() {
		self::init();
		self::set_paths_constants();
		self::set_base_constants();
		self::set_email_constants();
		self::set_cpt_constants();
		self::set_assets_constants();
		self::set_meta_field_constants();
		self::set_custom_tables();
		self::set_update_tables_constants();
		self::set_updater_constants();
		self::set_product_info_constants();
	}

	/**
	 * Set the plugin base constants.
	 */
	private static function set_base_constants() {

		// This is super important to check if the get_plugin_data function is already loaded or not.
		if ( ! function_exists( 'get_plugin_data' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}
		$plugin_data = get_plugin_data( BAF_DIR . BAF_ROOT_FILE );

		define( 'BAF_PLUGIN_VERSION', $plugin_data['Version'] ?? '1.0.0' );
		define( 'BAF_PLUGIN_TITLE', $plugin_data['Name'] ?? 'BWL Advanced FAQ Manager' );
		define( 'BAF_TRANSLATION_DIR', $plugin_data['DomainPath'] ?? '/lang/' );
		define( 'BAF_TEXT_DOMAIN', $plugin_data['TextDomain'] ?? '' );

		define( 'BAF_PLUGIN_FOLDER', 'bwl-advanced-faq-manager' );
		define( 'BAF_CURRENT_VERSION', BAF_PLUGIN_VERSION );
	}

	/**
	 * Set the plugin email constants.
	 */
	private static function set_email_constants() {
		define( 'BAF_A_FS_EMAIL_HEADER_TITLE', apply_filters( 'baf_a_fs_email_header_title', esc_html__( 'New FAQ Question', 'bwl-adv-faq' ) ) );
		define( 'BAF_A_FS_EMAIL_SUBJECT', apply_filters( 'baf_a_fs_email_subject', esc_html__( 'New FAQ submited!', 'bwl-adv-faq' ) ) );
		define( 'BAF_A_FS_REPLY_EMAIL', apply_filters( 'baf_a_fs_email_subject', 'no-reply@email.com' ) );
	}


	/**
	 * Set the plugin custom post type constants.
	 */
	private static function set_cpt_constants() {

		// User can change this from options panel.

		$rewrite_slug = self::$plugin_options[ BAF_CPT_CUSTOM_SLUG ] ?? 'bwl-advanced-faq';

		$show_in_rest = self::$plugin_options[ BAF_GUTENBERG_STATUS ] ?? 0;

		$publicly_queryable = self::$plugin_options[ BAF_DISABLE_SINGLE_FAQ_STATUS ] ?? 0;

		$sub_cat_enable_status = self::$plugin_options[ BAF_SUB_CAT_STATUS ] ?? 0;

		define( 'BAF_POST_TYPE', 'bwl_advanced_faq' );
		define( 'BAF_CPT_REWRITE_SLUG', $rewrite_slug );

		define( 'BAF_CPT_MENU_NAME', apply_filters( 'baf_menu_text', esc_html__( 'Advanced FAQ', 'bwl-adv-faq' ) ) );
		define( 'BAF_CPT_SINGULAR_NAME', apply_filters( 'baf_s_faq_text', esc_html__( 'FAQ', 'bwl-adv-faq' ) ) );
		define( 'BAF_CPT_PLURAL_NAME', apply_filters( 'baf_p_faq_text', esc_html__( 'FAQs', 'bwl-adv-faq' ) ) );
		define( 'BAF_CPT_QUERY_VAR', 'advanced_faq' );
		define( 'BAF_CPT_SLUG', 'advanced_faq_topics' );
		define( 'BAF_CPT_SHOW_IN_REST', $show_in_rest );
		// We are just reversing the value.
		// If the user set the disable single faq status to 1 then we will set the publicly queryable to false.
		define( 'BAF_CPT_PUBLICLY_QUERYABLE', ! ( $publicly_queryable ) );

		// By default we are setting the sub category status to 0.
		// If the user set the sub category status to 1 then we will set the sub category status to true.
		define( 'BAF_SUB_CAT_ENABLE_STATUS', $sub_cat_enable_status );

		define( 'BAF_TAX_CAT', 'advanced_faq_category' );
		define( 'BAF_TAX_CAT_TITLE', BAF_CPT_SINGULAR_NAME . ' ' . esc_html__( ' Category', 'bwl-adv-faq' ) );
		define( 'BAF_TAX_CAT_SLUG', BAF_CPT_REWRITE_SLUG . '-category' );
		define( 'BAF_TAX_CAT_QUERY_VAR', BAF_TAX_CAT );

		define( 'BAF_TAX_TOPIC', 'advanced_faq_topics' );
		define( 'BAF_TAX_TOPIC_TITLE', BAF_CPT_SINGULAR_NAME . ' ' . esc_html__( ' Topics', 'bwl-adv-faq' ) ); // phpcs:ignore
		define( 'BAF_TAX_TOPIC_SLUG', BAF_CPT_REWRITE_SLUG . '-topics' );
		define( 'BAF_TAX_TOPIC_QUERY_VAR', BAF_TAX_TOPIC );

		// Prefix
		define( 'PREFIX_BAF_CAT', 'baf_cat_' ); // Change this value in to 0 in Devloper mode :)
		define( 'PREFIX_BAF_TOPIC', 'baf_topics_' ); // Change this value in to 0 in Devloper mode :)

		// Filters
		define( 'BAF_MENU_TEXT', apply_filters( 'baf_menu_text', esc_html__( 'Advanced FAQ', 'bwl-adv-faq' ) ) );
		define( 'BAF_S_FAQ_TEXT', apply_filters( 'baf_s_faq_text', esc_html__( 'FAQ', 'bwl-adv-faq' ) ) );
		define( 'BAF_P_FAQ_TEXT', apply_filters( 'baf_p_faq_text', esc_html__( 'FAQs', 'bwl-adv-faq' ) ) );
	}

	/**
	 * Set the plugin paths constants.
	 */
	private static function set_paths_constants() {
		define( 'BAF_ROOT_FILE', 'bwl_advanced_faq_manager.php' );
		define( 'BAF_DIR', self::get_plugin_path() );
		define( 'BAF_FILE_PATH', BAF_DIR );
		define( 'BAF_URL', self::get_plugin_url() );
		define( 'BAF_CONTROLLER_DIR', BAF_DIR . 'includes/Controllers/' );
		define( 'BAF_VIEWS_DIR', BAF_DIR . 'includes/Views/' );
	}

	/**
	 * Set the plugin assets constants.
	 */
	private static function set_assets_constants() {
		define( 'BAF_STYLES_ASSETS_DIR', BAF_URL . 'assets/styles/' );
		define( 'BAF_SCRIPTS_ASSETS_DIR', BAF_URL . 'assets/scripts/' );
		define( 'BAF_LIBS_DIR', BAF_URL . 'libs/' );
	}

	/**
	 * Set the meta field constants.
	 */
	private static function set_meta_field_constants() {
		define( 'BWL_CMB_PREFIX', '_cmb_bptm_' );
		define( 'BPTM_DEFAULT_SIGN_TARGET', 50 ); // User sign target
		define( 'BPTM_META_USER_SIGN_TARGET', BWL_CMB_PREFIX . 'sign_target' ); // User sign target
		define( 'BPTM_META_USER_SIGN_COUNT', BWL_CMB_PREFIX . 'sign_lists' ); // Original user sign counts
		define( 'BPTM_META_MANUAL_SIGN_COUNT', BWL_CMB_PREFIX . 'manual_sign' ); // Manual user sign counts
	}

	/**
	 * Set the custom table names.
	 */
	private static function set_custom_tables() {
		global $wpdb;
		define( 'TABLE_BAF_LIKES_DATA', $wpdb->prefix . 'baf_likes_data' );
		define( 'TABLE_BAF_VIEWS_DATA', $wpdb->prefix . 'baf_views_data' );
		define( 'TABLE_BAF_SEARCH_KEYWORDS_DATA', $wpdb->prefix . 'baf_search_keywords_data' );
	}

	/**
	 * Set the update table constants.
     */
	private static function set_update_tables_constants() {
		// 1=Force Plugin To Recount Petition Data , 0= No Count.
		define( 'BPTM_UPDATE_DATA', 0 );
		// Just Change the version.
		define( 'BPTM_UPDATE_VERSION', '1_0_2' );
	}

	/**
	 * Set the updater constants.
	 */
	private static function set_updater_constants() {
		define( 'BAF_UPDATER_SLUG', BAF_PLUGIN_FOLDER . '/' . BAF_ROOT_FILE ); // phpcs:ignore
		define( 'BAF_PATH', BAF_DIR );
	}

	/**
	 * Set the product info constants.
	 */
	private static function set_product_info_constants() {
		define( 'BAF_PRODUCT_ID', '5007135' ); // Plugin codecanyon/themeforest Id.
		define( 'BAF_PRODUCT_INSTALLATION_TAG', 'baf_installation_' . str_replace( '.', '_', BAF_PLUGIN_VERSION ) );
		define( 'BAF_PRODUCT_URL', 'https://1.envato.market/baf-wp' ); // Plugin codecanyon URL.
		define( 'BAF_PURCHASE_VERIFIED_KEY', 'baf_purchase_verified' ); // old constant: BAF_PURCHASE_VERIFIED_KEY
		define( 'BAF_PURCHASE_INFO_KEY', 'baf_purchase_info' ); // old constant: BAF_PURCHASE_INFO_KEY
		define( 'BAF_PRODUCT_SUPPORT', 'https://codecanyon.net/item/bwl-advanced-faq-manager/5007135/support/contact' ); // old constant: BWL_BAF_AUTHOR_SUPPORT
		define( 'BAF_PRODUCT_YOUTUBE_PLAYLIST', 'https://www.youtube.com/playlist?list=PLxYTuQlgnCLqE7618FUZ1lrFJWTo3oRLi' ); // old constant: BWL_BAF_YOUTUBE_PLAYLIST
		define( 'BAF_PRODUCT_DOC', 'https://projects.bluewindlab.net/wpplugin/baf/doc' ); // old constant: BWL_BAF_DOC
		define( 'BAF_PRODUCT_AUTHOR_PROFILE', 'https://codecanyon.com/user/xenioushk' ); // old constant: BWL_BAF_AUTHOR_CC_PROFILE
		define( 'BAF_PRODUCT_VERIFIED_STATUS', (int) get_option( BAF_PURCHASE_VERIFIED_KEY ) );  // old constant: BWL_BAF_VERIFIED
		define( 'BAF_PRODUCT_INSTALLATION_DATE', 'baf_installation_date' );
	}
}
