<?php

/**
 * Initializes the BwlFaqManager plugin.
 *
 * @package BwlFaqManager
 */

namespace BwlFaqManager;

/**
 * Class Init
 *
 * Initializes and registers all the services for the BwlFaqManager plugin.
 *
 * @package BwlFaqManager
 */
class Init {


	/**
	 * Retrieves the list of services to be registered
	 *
	 * @since 2.0.6
	 */
	public static function get_services() {

		/**
		 * Add plugin required classes.
		 *
		 * @since 1.0.0
		 */

		$services = [
			Controllers\ExternalFaqForm\FaqSubmissionHandler::class,
			Controllers\Themes\FaqCustomThemes::class,
			Controllers\DashboardWidgets\PluginDashboardWidgets::class,
		];

		$service_classes = [
			'helpers'       => self::get_helper_classes(),
			'base'          => self::get_base_classes(),
			'meta'          => self::get_meta_classes(),
			'actions'       => self::get_action_classes(),
			'filters'       => self::get_filter_classes(),
			'cpt'           => self::get_cpt_classes(),
			// 'cmb'              => self::get_cmb_classes(),
			'shortcode'     => self::get_shortcode_classes(),
			// 'wpbakery'         => self::get_wpbakery_classes(),
			'notices'       => self::get_notices_classes(),
			'options_panel' => self::get_options_panel_classes(),
			// 'role_manager'     => self::get_role_manager_classes(),
			// 'template_manager' => self::get_template_manager_classes(),
		];

		foreach ( $service_classes as $service_class ) {
			$services = array_merge( $services, $service_class );
		}

		return $services;
	}

	/**
	 * Registered all the classes.
	 *
	 * @since 1.0.0
	 */
	public static function register_services() {

		if ( empty( self::get_services() ) ) {
			return;
		}

		foreach ( self::get_services() as $service ) {

			$service = self::instantiate( $service );

			if ( method_exists( $service, 'register' ) ) {
				$service->register();
			}
		}
	}

	/**
	 * Instantiate all the registered service classes.
	 *
	 * @param   class $service The class to instantiate.
	 * @author   Md Mahbub Alam Khan
	 * @return  object
	 * @since   1.1.0
	 */
	private static function instantiate( $service ) {
		return new $service();
	}

	/**
	 * Get Helper classes.
	 *
	 * @return array
	 * @note Ordering of the classes is important.
	 *    The classes should be ordered in a way that the dependencies are loaded first.
	 */
	private static function get_helper_classes() {
		$classes = [
			Helpers\OptionsPanelConstants::class,
			Helpers\SchedulerConstants::class,
			Helpers\PluginConstants::class,
			Helpers\Common::class,
		];
		return $classes;
	}

	/**
	 * Get Meta classes.
	 *
	 * @return array
	 */
	private static function get_meta_classes() {
		$classes = [
			Controllers\PluginMeta\MetaInfo::class,
		];
		return $classes;
	}

	/**
	 * Get Base classes.
	 *
	 * @return array
	 */
	private static function get_base_classes() {
		$classes = [
			Base\Enqueue::class,
			Base\AdminEnqueue::class,
			Base\Language::class,
			Base\PluginInstallation::class,
			Base\FrontendAjaxHandlers::class,
			Base\AdminAjaxHandlers::class,
			Base\AboutPluginRedirect::class,
			Base\PluginUpdate::class,
			Base\IncludePluginFiles::class,
		];
		return $classes;
	}

	/**
	 * Get Action classes.
	 *
	 * @return array
	 */
	private static function get_action_classes() {

		$classes = [
			Controllers\Actions\Admin\Scheduler::class,
		];
		return $classes;
	}

	/**
	 * Get Filter classes.
	 *
	 * @return array
	 */
	private static function get_filter_classes() {

		$classes = [
			Controllers\Filters\CategorySort::class,
			Controllers\Filters\Admin\CronScheduler::class,
		];
		return $classes;
	}

	/**
	 * Get CPT classes.
	 *
	 * @return array
	 */
	private static function get_cpt_classes() {
		$classes = [
			Controllers\Cpt\PluginCpt::class,
			Controllers\Cpt\CustomColumns::class,
			Controllers\Cpt\QuickBulkEdit::class,
			Controllers\Cpt\TaxonomyFilters::class,
		];
		return $classes;
	}

	/**
	 * Get Shortcode classes.
	 *
	 * @return array
	 */
	private static function get_shortcode_classes() {
		$classes = [
			Controllers\ShortcodeEditor\PluginScEditor::class,
			Controllers\Shortcodes\FaqSearchBox::class,
			Controllers\Shortcodes\FaqList::class,
			Controllers\Shortcodes\FaqAuthor::class,
			Controllers\Shortcodes\FaqSortedList::class,
			Controllers\Shortcodes\FaqExternalForm::class,
			Controllers\Shortcodes\FaqTabs::class,
		];
		return $classes;
	}

	/**
	 * Get Notices classes.
	 *
	 * @return array
	 */
	private static function get_notices_classes() {
		$classes = [
			Controllers\Notices\PluginNotices::class,
			Controllers\Notices\PluginNoticesAjaxHandler::class,
		];
		return $classes;
	}

	/**
	 * Get Options Panel classes.
	 *
	 * @return array
	 */
	private static function get_options_panel_classes() {
		$classes = [
			Controllers\OptionsPanel\SettingsMenu::class,
			Controllers\OptionsPanel\SettingsPage::class,
		];
		return $classes;
	}
}
