<?php
namespace BwlFaqManager\Traits;

trait ShortcodeEditorFieldsTraits {

	/**
	 * Text field
     *
     * @param array $args select field arguments.
     *
     * @return string
	 */
	public function get_text_field( $args = [] ) {

			$default_args = [
				'name'       => '',
				'id'         => '',
				'title'      => '',
				'class_name' => '',
				'value'      => '',
				'hints'      => '',
			];
			$args         = array_merge( $default_args, $args );

			// Extract the arguments
			extract( $args ); //phpcs:ignore

			$label_html = '';

			if ( ! empty( $title ) ) {
				$label_html = sprintf(
                    "<label for='%s'>%s</label>",
                    esc_attr( $id ),
                    esc_html( $title )
				);
			}
			return sprintf(
                "%s<input type='text' name='%s' id='%s' class='%s' value='%s'/>%s",
                $label_html,
                esc_attr( $name ),
                esc_attr( $id ),
                esc_attr( $class_name ),
                esc_attr( $value ),
                ( ! empty( $hints ) ) ? $this->get_field_hints( $hints ) : ''
            );
	}

	/**
     * Textarea field
     *
     * @param string $name       The name of the field.
     * @param string $id         The id of the field.
     * @param string $value      The value of the field.
     * @param string $placeholder Placeholder text.
     * @param string $class_name The class of the field.
     * @param string $hints      Hints for the field.
     *
     * @return string
     */
	public function get_textarea_field( $name, $id, $value = '', $placeholder = '', $class_name = 'large-text', $hints = '' ) {
		return sprintf(
            "<textarea name='%s' id='%s' class='%s' placeholder='%s'>%s</textarea>%s",
            esc_attr( $name ),
            esc_attr( $id ),
            esc_attr( $class_name ),
            esc_attr( $placeholder ),
            esc_textarea( $value ),
            ( ! empty( $hints ) ) ? $this->get_field_hints( $hints ) : ''
		);
	}

	/**
     * Checkbox field
     *
     * @param array $args select field arguments.
     *
     * @return string
     */
	public function get_checkbox_field( $args = [] ) {

			$default_args = [
				'name'       => '',
				'id'         => '',
				'title'      => '',
				'checked'    => false,
				'class_name' => '',
				'value'      => '',
				'hints'      => '',
			];
			$args         = array_merge( $default_args, $args );

			// Extract the arguments
			extract( $args ); //phpcs:ignore

			$label_html = '';

			if ( ! empty( $title ) ) {
				$label_html = sprintf(
                    "<label for='%s'>%s</label>",
                    esc_attr( $id ),
                    esc_html( $title )
				);
			}

			return sprintf(
                "%s<input type='checkbox' id='%s' name='%s' class='%s' value='%s' %s />%s",
                $label_html,
                esc_attr( $id ),
                esc_attr( $name ),
                esc_attr( $class_name ),
                esc_attr( $value ),
                checked( $checked, true, false ),
                ( ! empty( $hints ) ) ? $this->get_field_hints( $hints ) : ''
            );
	}

		/**
         * Radio Button field
         *
         * @param array $args select field arguments.
         *
         * @return string
         */
	public function get_radio_btn_field( $args = [] ) {

		$default_args = [
			'name'       => '',
			'id'         => '',
			'title'      => '',
			'checked'    => false,
			'class_name' => '',
			'value'      => '',
			'hints'      => '',
		];
		$args         = array_merge( $default_args, $args );

		// Extract the arguments
		extract( $args ); //phpcs:ignore

		return sprintf(
			"<input type='radio' id='%s' name='%s' class='%s' value='%s' %s />%s %s",
			esc_attr( $id ),
			esc_attr( $name ),
			esc_attr( $class_name ),
			esc_attr( $value ),
			checked( $checked, true, false ),
			esc_attr( $title ),
			( ! empty( $hints ) ) ? $this->get_field_hints( $hints ) : ''
		);
	}

	/**
	 * Select field
	 *
	 * @param array $args select field arguments.
	 *
	 * @return string
	 */
	public function get_select_field( $args = [] ) {

		$default_args = [
			'name'     => '',
			'id'       => '',
			'title'    => '',
			'options'  => [],
			'selected' => '',
			'hints'    => '',
		];
		$args         = array_merge( $default_args, $args );

		// Extract the arguments
		extract( $args ); //phpcs:ignore

		$label_html = '';

		if ( ! empty( $title ) ) {
			$label_html = sprintf(
                "<label for='%s'>%s</label>",
                esc_attr( $id ),
                esc_html( $title )
			);
		}

		$options_html = '';

		foreach ( $options as $value => $label ) {

			$value_data = ( is_array( $label ) ) ? $label['slug'] : $value;
			$label_data = ( is_array( $label ) ) ? $label['name'] : $label;

			$options_html .= sprintf(
                "<option value='%s'%s>%s</option>",
                esc_attr( $value_data ),
                selected( $selected, $value_data, false ),
                esc_html( $label_data )
			);
		}

		return sprintf("%s<select name='%s' id='%s'>%s</select>%s",
            $label_html,
            esc_attr( $name ),
            esc_attr( $id ),
            $options_html,
            ( ! empty( $hints ) ) ? $this->get_field_hints( $hints ) : ''
		);
	}


	/**
	 * Get field hints.
	 *
	 * @param string $hints Hints for the field.
	 *
	 * @return string
	 */
	private function get_field_hints( $hints ) {
		return sprintf(
            '<span class="bwl-opt-tooltip"><i class="dashicons dashicons-info"></i><span class="tooltip-text">%s</span></span>',
            esc_html( $hints )
		);
	}

	/**
	 * Get the dropdown options for boolean values.
	 *
	 * @param array $data Additional options to merge with the default options.
	 *
	 * @return array
	 */
	public function get_boolean_dropdown_options( $data = [] ) {

		$options = [
			'1' => esc_html__( 'Yes', 'default' ),
			'0' => esc_html__( 'No', 'default' ),
		];

		if ( ! empty( $data ) ) {
			$options = array_merge( $options, $data );
		}

		return $options;
	}
}
